/*
 *  SoapUI IntelliJ Plugin, copyright (C) 2006 eviware.com
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.eviware.soapui.intellij.plugin;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;
import org.jetbrains.annotations.NotNull;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.actions.SoapUIPreferencesAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.support.SwingToolHost;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.util.PanelBuilderRegistry;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.model.workspace.WorkspaceFactory;
import com.eviware.soapui.monitor.TestMonitor;
import com.eviware.soapui.monitor.support.TestMonitorListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.ui.Navigator;
import com.eviware.soapui.ui.NavigatorListener;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.support.DesktopListenerAdapter;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormFactory;
import com.eviware.x.impl.swing.SwingDialogs;
import com.eviware.x.impl.swing.SwingFileDialogs;
import com.eviware.x.impl.swing.SwingXFormDialogBuilder;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.openapi.wm.WindowManager;

/**
 * Main Plugin class, sets up SoapUI core and creates toolwindows 
 */

public class SoapUIPlugin implements ProjectComponent
{
   private Project myProject;

   private final static Logger log = Logger.getLogger(SoapUIPlugin.class);
   private static final String WORKSPACE_WINDOW_ID = "soapUI Navigator";
   private Navigator navigator;
   private Workspace workspace;
   private static final String LOG_WINDOW_ID = "soapUI Log";
   private JPanel overviewPanel;
   private static final String DEFAULT_WORKSPACE_FILE = "intellij-soapui-workspace.xml";

   public SoapUIPlugin( Project project )
   {
       myProject = project;

       ClassLoader loader = Thread.currentThread().getContextClassLoader();
       Thread.currentThread().setContextClassLoader( SoapUI.class.getClassLoader() );

       try
       {
          UISupport.setToolHost( new SwingToolHost() );
          XFormFactory.setDelegate( new XFormFactory.Delegate()
          {
             public XFormDialogBuilder createDialogBuilder( String name )
             {
                return new SwingXFormDialogBuilder( name );
             }
          } );

          URL log4jUrl = SoapUI.class.getResource( "/soapui-log4j.xml" );
          if( log4jUrl == null )
          {
            File log4jconfig = new File( "soapui-log4j.xml" );
            if( log4jconfig.exists())
            {
                System.out.println( "Configuring log4j from [" + log4jconfig + "]" );
                DOMConfigurator.configureAndWatch( log4jconfig.getAbsolutePath(), 5000 );
            }
          }
          else
          {
            System.out.println( "Configuring log4j from [" + log4jUrl + "]" );
            DOMConfigurator.configure( log4jUrl );
          }

           // stupid initializer
           new SoapUIPreferencesAction();
       }
       catch( Exception e )
       {
          e.printStackTrace();
       }
       finally
       {
           Thread.currentThread().setContextClassLoader( loader );
       }
   }

   public void projectOpened()
   {
      initToolWindow();
      UISupport.setMainFrame( WindowManager.getInstance().suggestParentWindow( myProject ) );
      UISupport.setDialogs( new SwingDialogs( UISupport.getMainFrame() ) );
      UISupport.setFileDialogs( new SwingFileDialogs( UISupport.getMainFrame() ) );
   }

   public void projectClosed()
   {
      unregisterToolWindow();
   }

   public void initComponent()
   {
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      Thread.currentThread().setContextClassLoader( SoapUI.class.getClassLoader() );

      try
      {
         SoapUI soapUI = new SoapUI();
         soapUI.buildLogPanel( true, "soapui");

         workspace = WorkspaceFactory.getInstance().openWorkspace( new String[]{ System.getProperty( "user.home" ) +
                 File.separatorChar + DEFAULT_WORKSPACE_FILE } );
         navigator = new Navigator( workspace );
         navigator.addNavigatorListener( new InternalNavigatorListener() );

         IntellijSoapUIDesktop desktop = new IntellijSoapUIDesktop( workspace, myProject );
         desktop.addDesktopListener( new DesktopListenerAdapter()
         {
            public void desktopPanelSelected( DesktopPanel desktopPanel )
            {
               ModelItem modelItem = desktopPanel.getModelItem();
               if( modelItem != null )
               {
                  navigator.selectModelItem( modelItem );
               }
            }
         } );

         SoapUI.setNavigator( navigator );
         SoapUI.setDesktop( desktop );

         TestMonitor testMonitor = SoapUI.getTestMonitor();

		 testMonitor.addTestMonitorListener( new InternalTestMonitorListener() );
         testMonitor.init( workspace );
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      finally
      {
          Thread.currentThread().setContextClassLoader( loader );
      }
   }

   public void disposeComponent()
   {
      if( workspace != null )
      {
         workspace.save( false );
         try
         {
            SoapUI.saveSettings();
         }
         catch( IOException e )
         {
            e.printStackTrace();
         }
      }
   }

   @NotNull
   public String getComponentName()
   {
      return "SoapUI Plugin";
   }

   private void initToolWindow()
   {
      ToolWindowManager toolWindowManager = ToolWindowManager.getInstance( myProject );

      toolWindowManager.registerToolWindow( WORKSPACE_WINDOW_ID, buildMainPanel(), ToolWindowAnchor.LEFT );
      toolWindowManager.registerToolWindow( LOG_WINDOW_ID, SoapUI.getLogMonitor(), ToolWindowAnchor.BOTTOM );
   }

   private void unregisterToolWindow()
   {
      ToolWindowManager toolWindowManager = ToolWindowManager.getInstance( myProject );
      toolWindowManager.unregisterToolWindow( WORKSPACE_WINDOW_ID );
      toolWindowManager.unregisterToolWindow( LOG_WINDOW_ID );
   }

   private JComponent buildMainPanel()
   {
      JSplitPane splitPane = UISupport.createVerticalSplit();

      splitPane.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );

      splitPane.setTopComponent( navigator );
      splitPane.setBottomComponent( buildOverviewPanel() );
      splitPane.setDividerLocation( 0.7 );
      splitPane.setResizeWeight( 0.7 );
      return splitPane;
   }

   private Component buildOverviewPanel()
   {
      overviewPanel = new JPanel( new BorderLayout() );
      return overviewPanel;
   }

   private void setOverviewPanel( Component panel )
   {
      if( overviewPanel.getComponentCount() == 0 && panel == null )
      {
         return;
      }

      overviewPanel.removeAll();
      if( panel != null )
      {
         overviewPanel.add( panel, BorderLayout.CENTER );
      }
      overviewPanel.revalidate();
      overviewPanel.repaint();
   }

   public class InternalNavigatorListener implements NavigatorListener
   {
      public void nodeSelected( SoapUITreeNode treeNode )
      {
      	if( treeNode == null )
			{
				setOverviewPanel( null );
			}
			else
			{
				PanelBuilder<ModelItem> panelBuilder = PanelBuilderRegistry.getPanelBuilder( treeNode.getModelItem() );
				if( panelBuilder != null && panelBuilder.hasOverviewPanel() )
				{
					setOverviewPanel( panelBuilder.buildOverviewPanel( treeNode.getModelItem() ) );
				}
				else
				{
					setOverviewPanel( null );
				}
			}
      }
   }

    private static final class InternalTestMonitorListener extends TestMonitorListenerAdapter
    {
            private Set<LoadTestRunner> loadTestRunners = new HashSet<LoadTestRunner>();

            public void loadTestStarted(LoadTestRunner runner)
            {
                if( loadTestRunners.isEmpty() )
                {
                    log.info( "Disabling httpclient and groovy logs during loadtests" );
                    Logger.getLogger( "httpclient.wire" ).setLevel( Level.OFF );
                    Logger.getLogger( "groovy.log" ).setLevel( Level.OFF );
                }

                loadTestRunners.add( runner );
            }

            public void loadTestFinished(LoadTestRunner runner)
            {
                loadTestRunners.remove( runner );

                if( loadTestRunners.isEmpty() )
                {
                    Logger.getLogger( "httpclient.wire" ).setLevel( Level.DEBUG );
                    Logger.getLogger( "groovy.log" ).setLevel( Level.DEBUG );
                    log.info( "Enabled httpclient and groovy logs after loadtests" );
                }
            }
        }

}
